/*
 * Disseny i implementaci d'un marc de treball de presentaci per a aplicacions J2EE
 * 
 * Universitat Oberta de Catalunya (UOC)
 * 
 * Projecte de Final de Carrera - PFC - J2EE
 *
 * Alumne : Luis Bravo Fuentes
 * 
 * Consultor: Josep Maria Camps Riba
 */
package edu.uoc.fw.core.config;

import java.io.IOException;
import java.io.InputStream;
import java.util.HashMap;
import java.util.Map;
import java.util.Set;

import javax.servlet.ServletConfig;
import javax.servlet.ServletContext;

import org.apache.commons.digester3.Digester;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.xml.sax.SAXException;

import edu.uoc.fw.core.exceptions.ConfigurationException;

/**
 * Gestiona la configuraci de laplicaci. 
 * Aquesta configuraci es carregar a partir del fitxer de configuraci de laplicaci que utilitza el framework. <br>
 * Implementaci del patr Singleton, solament existir una instancia de la classe per a tota l'aplicaci.
 *
 * @author Luis Bravo Fuentes
 * @version 1.0
 */
public class Configuration {

	static Log log = LogFactory.getLog(Configuration.class);

	private static final String CONFIGURATION_FILE_PARAM 	= "configuration-file";
	private static final String CONFIGURATION_FILE_NAME 	= "/WEB-INF/framework-configuration.xml";
	
	private static Configuration instance;
	
	/**
	 * Resultats globals per a la navegaci. No caldr configurar-los en els Command
	 */
	private Map<String, ResultConfig> globalResults = new HashMap<String, ResultConfig>();

	/**
	 * Configuraci dels Command de l'aplicaci
	 */
	private Map<String, CommandConfig> commands = new HashMap<String, CommandConfig>();

	/**
	 * Configuraci dels beans de formulari (Form) de l'aplicaci
	 */
	private Map<String, FormConfig> forms = new HashMap<String, FormConfig>();

	/**
	 * Configuraci dels fitxers de properties per a suport del multiidioma
	 */
	private ResourceBundleConfig resourceBundleConfig = ResourceBundleConfig.getInstance();	
	
	/**
	 * Constructor de la classe, indica el fitxer que contindr la configuraci de l'aplicaci
	 *
	 * @param config
	 * @param context
	 */
	private Configuration(ServletContext context, ServletConfig config) throws ConfigurationException {
		String configurationFile = config.getInitParameter(CONFIGURATION_FILE_PARAM);
		if (configurationFile == null) {
			configurationFile = CONFIGURATION_FILE_NAME;
		}
		digesterConfigFile(context, configurationFile);
	}
	

	/**
	 * Es retorna la instancia de <code>Configuration</code> que s'utilitzar a l'aplicaci.
	 *  
	 * @return Instncia de la classe <code>Configuration</code> que cont la configuraci.
	 * @throws ConfigurationException Si no hi ha cap instncia de <code>Configuration</code>
	 */
	public static Configuration getInstance() throws ConfigurationException {
		if (instance == null) {
			throw new ConfigurationException("No s'ha carregat el fitxer de configuraci");
		}
		return instance;
	}
	
	/**
	 * Inicialitza la configuraci. Crea la instancia de <code>Configuration</code> que s'utilitzar a l'aplicaci.
	 * Implementaci del patr Singleton.
	 * 
	 * @param context Contexte del servlet que s'utilitzar per a crear l'objecte i carregar la configuraci.
	 * @param config Configuraci del servlet que s'utilitzar per a crear l'objecte i carregar la configuraci.
	 * @return Instncia de la classe <code>Configuration</code> que cont la configuraci.
	 */
	public static Configuration initConfiguration(ServletContext context, ServletConfig config) throws ConfigurationException {
		if (instance == null) {
			instance = new Configuration(context, config);
		}
		return instance;
	}

	/**
	 * S'informa la classe amb les dades que provenen del fitxer de configuraci de l'aplicaci. <br>
	 * S'utilitza <code>Apache Digester</code> per a tractar aquest fitxer XML.
	 * 
	 * @param context Contexte del servlet.
	 * @param fileName Nom del fitxer de configuraci.
	 * @return Instncia de la classe <code>Configuration</code> amb les dades del fitxer de configuraci XML indicat.
	 */
	public Configuration digesterConfigFile(ServletContext context, String fileName) throws ConfigurationException {

		InputStream input = context.getResourceAsStream(fileName);
		
		if (input == null) {
			throw new ConfigurationException("No s'ha trobat el fitxer de configuraci: " + fileName);
		}
		
		Digester digester = new Digester();
		digester.setValidating(false);
		digester.push(this);

		digester.addSetProperties("config");
		digester.addObjectCreate("config/global-results/result", ResultConfig.class);
		digester.addSetProperties("config/global-results/result");
		digester.addSetNext("config/global-results/result", "addGlobalResults");

		digester.addObjectCreate("config/forms/form", FormConfig.class);
		digester.addSetProperties("config/forms/form");
		digester.addSetNext("config/forms/form", "addForm");

		digester.addObjectCreate("config/commands/command", CommandConfig.class);
		digester.addSetProperties("config/commands/command");
		digester.addSetNext("config/commands/command", "addCommand");

		digester.addObjectCreate("config/commands/command/result", ResultConfig.class);
		digester.addSetProperties("config/commands/command/result");
		digester.addSetNext("config/commands/command/result", "addResult");

		digester.addBeanPropertySetter("config/locales/files", "filePattern");
		digester.addBeanPropertySetter("config/locales/default", "defaultLanguage");
		
        digester.addCallMethod("config/locales/language", "addLanguage", 1);
        digester.addCallParam("config/locales/language", 0);
		
		Configuration configuration;
	
		try {
			configuration = (Configuration) digester.parse(input);

		} catch (IOException e) {
			throw new ConfigurationException("Error al carregar el fitxer de configuraci: " + fileName, e);

		} catch (SAXException e) {
			throw new ConfigurationException("Error al tractar el fitxer de configuraci: " + fileName, e);

		} finally {
			try {
				input.close();

			} catch (IOException e) {
				throw new ConfigurationException("Error al tancar el fitxer de configuraci: " + fileName, e);
			}
		}

		log.info(configuration);

		return configuration;
	}

	/**
	 * Retorna un String amb les dades de la instncia de la classe <code>Configuration</code>.
	 */
	@Override
	public String toString() {
		Set<String> keyGlobalResults = globalResults.keySet();
		Set<String> keyCommand = commands.keySet();
		Set<String> keyForm = forms.keySet();
		
		String str = "Configuration {";
		str += "global-results: {";
		for (String key : keyGlobalResults) {
			str += "\n{key: \"" + key + "\", value: " + globalResults.get(key) + "} ";
		}
		str += "}, commands: {";
		for (String key : keyCommand) {
			str += "\n{key: \"" + key + "\", value: " + commands.get(key) + "} ";
		}
		str += "}, forms: {";
		for (String key : keyForm) {
			str += "\n{key: \"" + key + "\", value: " + forms.get(key) + "} ";
		}
		str += "}, locale: " + resourceBundleConfig + "}";
		
		return str;
	}
	/*
	 * Mtodes necessaris per a que Apache Digester informi les dades del fitxer de configuraci a la instncia de la classe.
	 */
	
	/**
	 * Afegeix una instncia de <code>ResultConfig</code>, la configuraci d'un resultat, a la hash de resultats globals de la configuraci.
	 * 
	 * @param resultConfig
	 */
	public void addGlobalResults(ResultConfig resultConfig) {
		globalResults.put(resultConfig.getName(), resultConfig);
	}

	/**
	 * Afegeix una instncia de <code>FormConfig</code>, la configuraci d'un formulari, a la hash de formularis de la configuraci.
	 * 
	 * @param formConfig
	 */
	public void addForm(FormConfig formConfig) {
		forms.put(formConfig.getName(), formConfig);
	}

	/**
	 * Afegeix una instncia de <code>CommandConfig</code>, la configuraci d'un command, a la hash de commands de la configuraci.
	 * 
	 * @param commandConfig
	 */
	public void addCommand(CommandConfig commandConfig) {
		commands.put(commandConfig.getName(), commandConfig);
	}
	
	/**
	 * Afegeix un codi d'idioma a la configuraci dels recursos 
	 * 
	 * @param language
	 */
	public void addLanguage(String language) {
		resourceBundleConfig.addLanguage(language);
	}
	
	/**
	 * Retorna la configuraci d'un resultat de l'aplicaci segons el seu nom.
	 * 
	 * @param resultConfigName Nom del resultat que es vol recuperar.
	 * @return Objecte de la classe <code>ResultConfig</code> amb la configuraci del resultat.
	 */
	public ResultConfig getGlobalResult(String resultConfigName) {
		return globalResults.get(resultConfigName);
	}

	/**
	 * Retorna la configuraci d'un command de l'aplicaci segons el seu nom.
	 * 
	 * @param commandName Nom del command que es vol recuperar.
	 * @return Objecte de la classe <code>ResultConfig</code> amb la configuraci del command.
	 */
	public CommandConfig getCommandConfig(String commandName) {
		return commands.get(commandName);
	}

	/**
	 * Retorna la configuraci d'un formulari de l'aplicaci segons el seu nom.
	 * 
	 * @param formName Nom del formulari que es vol recuperar.
	 * @return Objecte de la classe <code>FormConfig</code> amb la configuraci del formulari.
	 */
	public FormConfig getFormConfig(String formName) {
		return forms.get(formName);
	}
	
	/**
	 * S'estableix el patr que s'utilitzar a l'hora de recuperar els fitxers properties per al multiidioma
	 * 
	 * @param filePattern Patr a utilitzar.
	 */
	public void setFilePattern(String filePattern) {
		resourceBundleConfig.setFilePattern(filePattern);
	}
	
	/**
	 * S'estableix l'idioma per defecte de l'aplicaci.
	 * 
	 * @param defaultLanguage Codi de l'idioma per defecte.
	 */
	public void setDefaultLanguage(String defaultLanguage) {
		resourceBundleConfig.setDefaultLanguage(defaultLanguage);
	}
}