/*
 * Disseny i implementaci d'un marc de treball de presentaci per a aplicacions J2EE
 * 
 * Universitat Oberta de Catalunya (UOC)
 * 
 * Projecte de Final de Carrera - PFC - J2EE
 *
 * Alumne : Luis Bravo Fuentes
 * 
 * Consultor: Josep Maria Camps Riba
 */
package edu.uoc.fw.core.controller;

import java.lang.reflect.InvocationTargetException;
import java.util.Map;
import java.util.Map.Entry;

import org.apache.commons.beanutils.BeanUtils;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import edu.uoc.fw.core.command.Command;
import edu.uoc.fw.core.config.CommandConfig;
import edu.uoc.fw.core.config.Configuration;
import edu.uoc.fw.core.config.FormConfig;
import edu.uoc.fw.core.config.ResultConfig;
import edu.uoc.fw.core.context.ContextObject;
import edu.uoc.fw.core.exceptions.ConfigurationException;
import edu.uoc.fw.core.form.Form;

/**
 * Centralitza la gesti i el procs de les peticions.
 * Implementaci del patr Application Controler i el patr Singleton: solament existir una instancia de la classe per a tota l'aplicaci.
 * 
 * @author Luis Bravo Fuentes
 * @version 1.0
 */
public class ApplicationController {

	static Log log = LogFactory.getLog(ApplicationController.class);

	/**
	 * S'obt la configuraci del command amb el nom indicat. S'obt l'objecte CommandConfig. 
	 * 
	 * @param commandName El nom del Command del que s'obtindr la configuraci
	 * @return Configuraci del Command
	 * @throws ConfigurationException
	 */
	private static CommandConfig getCommandConfiguration(String commandName) throws ConfigurationException {
	
		CommandConfig commandConfig = Configuration.getInstance().getCommandConfig(commandName);
	
		if (commandConfig == null) {
			throw new ConfigurationException("No s'ha trobat el command al fitxer de configuraci. Command: " + commandName);
		}
		return commandConfig;
	}
	
	/**
	 * S'obt la configuraci del form amb el nom indicat. S'obt l'objecte FormConfig.
	 * 
	 * @param formName El nom del formulari del que s'obtindr la configuraci
	 * @return Configuraci del Form
	 * @throws ConfigurationException
	 */
	private static FormConfig getFormConfiguration(String formName) throws ConfigurationException {
	
		FormConfig formConfig = Configuration.getInstance().getFormConfig(formName);
	
		if (formConfig == null) {
			throw new ConfigurationException("No s'ha trobat el form al fitxer de configuraci. Form: " + formName);
		}
		return formConfig;
	}
	
	/**
	 * S'obt al configuraci del resultat del Command indicat. Segons el resultat indicat, es recupera el ResultConfig de la 
	 * configuraci del Command
	 * 
	 * @param commandConfig Configuraci del Command de la que es vol recuperar la configuraci del resultat
	 * @param resultName Nom del resultat
	 * @return Configuraci del resultat per al Command indicat en la configuraci del Command
	 * @throws ConfigurationException
	 */
	private static ResultConfig getResultCommand(CommandConfig commandConfig, String resultName) throws ConfigurationException {
		
		// S'obt el result del command
		ResultConfig resultConfig = commandConfig.getResultByName(resultName);
		
		// Si no es troba, s'obt dels results globals
		if (resultConfig == null) {
			resultConfig = Configuration.getInstance().getGlobalResult(resultName);
			
			// Si no es troba als results globals, es dona una exception
			if (resultConfig == null) {
				throw new ConfigurationException("No s'ha troba el result " + resultName + " per al command: " + commandConfig.getName());
			}
		}
		
		return resultConfig;
	}

	/**
	 * S'informa el form indicat amb els parmetres de la petici
	 * 
	 * @param request
	 * @param formName
	 * @param form
	 */
	private static void populateForm(ContextObject context, String formName, Form form) {

		// Es recuperen els parmetres de la petici
		Map<String, String> parameters = context.getParametersMap();

		if (parameters.size() > 0) {
			// S'itera per a cada parmetre de la hash
			for (Entry<String, String> parameter : parameters.entrySet()) {
				String propertyName = parameter.getKey();
				// Es treu el nom del formulari dels parmetres que ho tenen
				// p.e.: dels parmetres de la forma nomDelFormulari.nomDelAtribut, propertyName valdr nomDelAtribut
				if (parameter.getKey().contains(formName + ".")) {
					propertyName = parameter.getKey().substring(parameter.getKey().indexOf(".") + 1);
				}
	
				try {
					// S'estableix el valor dels atributs de l'objecte form
					BeanUtils.setProperty(form, propertyName, parameter.getValue());
	
				} catch (IllegalAccessException e) {
					log.info("Propietat no trobada al form: " + formName);
	
				} catch (InvocationTargetException e) {
					log.info("Propietat no trobada al form: " + formName);
				}
			}
		}
	}

	/**
	 * Es crea una instancia de la classe Form indicada a l'objecte CommandConfig.
	 * 
	 * @param context Context de La petici
	 * @param commandConfig Configuraci de l'objecte de la classe Command que es crear
	 * @return Una instancia de la classe Form indicada a la configuraci
	 * @throws ConfigurationException
	 */
	private static Form createForm(ContextObject context, CommandConfig commandConfig) throws ConfigurationException {

		String formName = commandConfig.getForm();

		// S'obt la configuraci del Form
		FormConfig formConfig = getFormConfiguration(formName);

		// S'obt l'mbit del Form per a aquest Command
		String scope = commandConfig.getScope();

		Form form = null;
		
		// Si l'mbit del formulari s la sesi
		if (CommandConfig.Scope.SESSION.equals(scope)) {
			// Es recupera la instancia del Form de la sessi
			
			form = (Form) context.getAttributeSession(formName);

			// Si no est troba es crea i es guarda a la sessi
			if (form == null) {
				// Es crea la instancia del Form segons la configuraci
				form = formConfig.createForm();
				// Es guarda a la sessi
				context.setAttributeSession(formName, form);
			}

		} else {
			// Si l'mbit del formulari no s la sesi, es crea
			form = formConfig.createForm();
		}
		// S'informa la instancia del Form amb els parmetres de la petici
		populateForm(context, formName, form);

		// S'informa la petici amb el Form
		context.setAttributeRequest(formName, form);

		return form;
	}
	
	/**
	 * S'executa el command. Es crida al mtode <code>execute</code> de
	 * l'objecte de la classe <code>Command</code> que s'ha indicat al fitxer de
	 * configuraci
	 * 
	 * @param context Context de la petici
	 * @return
	 * @throws ConfigurationException
	 */
	protected static ResultConfig execute(ContextObject context) throws ConfigurationException {
		
		// S'obt el nom del command invocat
		String commandName = context.getServletPath();

		// S'obt la configuraci del command
		CommandConfig commandConfig = getCommandConfiguration(commandName);		
				
		// Es crea la clase command a partir de la seva configuraci
		Command command = commandConfig.createCommand();
		
		Form form = createForm(context, commandConfig);

		String resultName = "";
		
		//S'executa el mtode validate del command
		if (command.validate(context, form)) {
			// Si valida, s'executar el command

			// S'obt el result de l'execuci del command
			resultName = command.execute(context, form);
						
		} else {
			// Si no s'executa el command per un error de validaci, es torna a carregar la pgina d'entrada del formulari
			resultName = Command.INPUT_VIEW;

		}
		
		// S'incorporen els missatges a la petici
		context.setAttributeRequest(Command.COMMAND_MESSAGES, command.getMessages());

		// Amb el nom del result, s'obt el result del command o dels results globals
		ResultConfig resultConfig = getResultCommand(commandConfig, resultName);

		return resultConfig;
	}
}